@@ -20,6 +20,7 @@
  */
 
 #include "terminal-intl.h"
+#include "terminal.h"
 
 #include "encoding.h"
 
@@ -691,49 +692,12 @@ terminal_encoding_dialog_new (GtkWindow *transient_parent)
   GtkTreeIter parent_iter;
   GtkTreeSelection *selection;
   GtkWidget *dialog;
-  
-  if (g_file_test ("./"TERM_GLADE_FILE,
-                   G_FILE_TEST_EXISTS))
-    {
-      /* Try current dir, for debugging */
-      xml = glade_xml_new ("./"TERM_GLADE_FILE,
-                           "encodings-dialog",
-                           GETTEXT_PACKAGE);
-    }
-  else
-    {
-      xml = glade_xml_new (TERM_GLADE_DIR"/"TERM_GLADE_FILE,
-                           "encodings-dialog",
-                           GETTEXT_PACKAGE);
-    }
 
+  xml = terminal_util_load_glade_file (TERM_GLADE_FILE,
+                                       "encodings-dialog",
+                                       transient_parent);
   if (xml == NULL)
-    {
-      static GtkWidget *no_glade_dialog = NULL;
-          
-      if (no_glade_dialog == NULL)
-        {
-          no_glade_dialog =
-            gtk_message_dialog_new (transient_parent,
-                                    GTK_DIALOG_DESTROY_WITH_PARENT,
-                                    GTK_MESSAGE_ERROR,
-                                    GTK_BUTTONS_CLOSE,
-                                    _("The file \"%s\" is missing. This indicates that the application is installed incorrectly, so the encodings dialog can't be displayed."),
-                                    TERM_GLADE_DIR"/"TERM_GLADE_FILE);
-                                        
-          g_signal_connect (G_OBJECT (no_glade_dialog),
-                            "response",
-                            G_CALLBACK (gtk_widget_destroy),
-                            NULL);
-          
-          g_object_add_weak_pointer (G_OBJECT (no_glade_dialog),
-                                     (void**)&no_glade_dialog);
-        }
-
-      gtk_window_present (GTK_WINDOW (no_glade_dialog));
-
-      return NULL;
-    }  
+    return NULL;
 
   /* The dialog itself */
   dialog = glade_xml_get_widget (xml, "encodings-dialog");
@@ -953,15 +917,17 @@ terminal_encoding_init (GConfClient *conf)
 
   /* Initialize the sample text with all of the printing ASCII characters
    * from space (32) to the tilde (126), 95 in all. */ 
-  for (i = 0; i < sizeof(ascii_sample); i++) 
-    {
-      ascii_sample[i] = i + 32;
-    }
-  ascii_sample[sizeof(ascii_sample) - 1] = '\0';
+  for (i = 0; i < sizeof (ascii_sample); i++) 
+    ascii_sample[i] = i + 32;
 
+  ascii_sample[sizeof(ascii_sample) - 1] = '\0';
+  
   i = 0;
   while (i < TERMINAL_ENCODING_LAST)
     {
+      bytes_read = 0;
+      bytes_written = 0;
+      
       g_assert (encodings[i].index == i);
 
       /* Translate the names */
@@ -970,18 +936,20 @@ terminal_encoding_init (GConfClient *conf)
       /* Test that the encoding is a proper superset of ASCII (which naive
        * apps are going to use anyway) by attempting to validate the text
        * using the current encoding.  This also flushes out any encodings
-       * which the underlying GIConv implementation can't support. */
-      converted = g_convert (ascii_sample, sizeof(ascii_sample) - 1,
+       * which the underlying GIConv implementation can't support.
+       */
+      converted = g_convert (ascii_sample, sizeof (ascii_sample) - 1,
 		             encodings[i].charset, encodings[i].charset,
 			     &bytes_read, &bytes_written, NULL);
-
+      
       /* The encoding is only valid if ASCII passes through cleanly. */
       if (i == TERMINAL_ENCODING_CURRENT_LOCALE)
         encodings[i].valid = TRUE;
       else
-        encodings[i].valid = (bytes_read == (sizeof(ascii_sample) - 1)) &&
-			     (converted != NULL) &&
-			     (strcmp(converted, ascii_sample) == 0);
+        encodings[i].valid =
+          (bytes_read == (sizeof (ascii_sample) - 1)) &&
+          (converted != NULL) &&
+          (strcmp (converted, ascii_sample) == 0);
 
 #ifdef DEBUG_ENCODINGS
       if (!encodings[i].valid)
@@ -994,9 +962,7 @@ terminal_encoding_init (GConfClient *conf)
 
       /* Discard the converted string. */
       if (converted != NULL)
-        {
-          g_free (converted);
-        }
+        g_free (converted);
 
       ++i;
     }
