@@ -22,6 +22,7 @@
 #include "terminal-intl.h"
 #include "terminal-accels.h"
 #include "terminal-profile.h"
+#include "terminal.h"
 #include <string.h>
 #include <glade/glade.h>
 #include "eggcellrendererkeys.h"
@@ -45,6 +46,9 @@
 #define KEY_SET_TERMINAL_TITLE CONF_KEYS_PREFIX"/set_window_title"
 #define PREFIX_KEY_SWITCH_TO_TAB CONF_KEYS_PREFIX"/switch_to_tab_"
 #define KEY_HELP CONF_KEYS_PREFIX"/help"
+#define KEY_ZOOM_IN CONF_KEYS_PREFIX"/zoom_in"
+#define KEY_ZOOM_OUT CONF_KEYS_PREFIX"/zoom_out"
+#define KEY_ZOOM_NORMAL CONF_KEYS_PREFIX"/zoom_normal"
 
 typedef struct
 {
@@ -94,6 +98,12 @@ static KeyEntry view_entries[] =
     KEY_TOGGLE_MENUBAR, ACCEL_PATH_TOGGLE_MENUBAR, 0, 0, NULL, FALSE },
   { N_("Full Screen"),
     KEY_FULL_SCREEN, ACCEL_PATH_FULL_SCREEN, 0, 0, NULL, FALSE },
+  { N_("Zoom In"),
+    KEY_ZOOM_IN, ACCEL_PATH_ZOOM_IN, 0, 0, NULL, FALSE },
+  { N_("Zoom Out"),
+    KEY_ZOOM_OUT, ACCEL_PATH_ZOOM_OUT, 0, 0, NULL, FALSE },
+  { N_("Normal Size"),
+    KEY_ZOOM_NORMAL, ACCEL_PATH_ZOOM_NORMAL, 0, 0, NULL, FALSE }
 };
 
 static KeyEntry terminal_entries[] =
@@ -1070,49 +1080,12 @@ terminal_edit_keys_dialog_new (GtkWindow *transient_parent)
   GtkTreeViewColumn *column;
   GtkTreeIter parent_iter;
 
-  if (g_file_test ("./"TERM_GLADE_FILE,
-                   G_FILE_TEST_EXISTS))
-    {
-      /* Try current dir, for debugging */
-      xml = glade_xml_new ("./"TERM_GLADE_FILE,
-                           "keybindings-dialog",
-                           GETTEXT_PACKAGE);
-    }
-  else
-    {
-      xml = glade_xml_new (TERM_GLADE_DIR"/"TERM_GLADE_FILE,
-                           "keybindings-dialog",
-                           GETTEXT_PACKAGE);
-    }
-
+  xml = terminal_util_load_glade_file (TERM_GLADE_FILE,
+                                       "keybindings-dialog",
+                                       transient_parent);
   if (xml == NULL)
-    {
-      static GtkWidget *no_glade_dialog = NULL;
-          
-      if (no_glade_dialog == NULL)
-        {
-          no_glade_dialog =
-            gtk_message_dialog_new (transient_parent,
-                                    GTK_DIALOG_DESTROY_WITH_PARENT,
-                                    GTK_MESSAGE_ERROR,
-                                    GTK_BUTTONS_CLOSE,
-                                    _("The file \"%s\" is missing. This indicates that the application is installed incorrectly, so the keybindings dialog can't be displayed."),
-                                    TERM_GLADE_DIR"/"TERM_GLADE_FILE);
-                                        
-          g_signal_connect (G_OBJECT (no_glade_dialog),
-                            "response",
-                            G_CALLBACK (gtk_widget_destroy),
-                            NULL);
-
-          g_object_add_weak_pointer (G_OBJECT (no_glade_dialog),
-                                     (void**)&no_glade_dialog);
-        }
-
-      gtk_window_present (GTK_WINDOW (no_glade_dialog));
-
-      return NULL;
-    }
-
+    return NULL;
+  
   w = glade_xml_get_widget (xml, "disable-mnemonics-checkbutton");
   living_mnemonics_checkbuttons = g_slist_prepend (living_mnemonics_checkbuttons,
                                                    w);
