@@ -68,6 +68,7 @@ enum {
   PROFILE_SET,
   TITLE_CHANGED,
   SELECTION_CHANGED,
+  ENCODING_CHANGED,
   LAST_SIGNAL
 };
 
@@ -92,6 +93,9 @@ static void terminal_screen_widget_child_died        (GtkWidget      *term,
 static void terminal_screen_widget_selection_changed (GtkWidget      *term,
                                                       TerminalScreen *screen);
 
+static void terminal_screen_widget_encoding_changed  (GtkWidget      *term,
+                                                      TerminalScreen *screen);
+
 static void terminal_screen_setup_dnd                (TerminalScreen *screen);
 
 static void reread_profile (TerminalScreen *screen);
@@ -217,6 +221,10 @@ terminal_screen_init (TerminalScreen *screen)
                                              G_CALLBACK (terminal_screen_widget_selection_changed),
                                              screen);
 
+  terminal_widget_connect_encoding_changed (screen->priv->term,
+                                            G_CALLBACK (terminal_screen_widget_encoding_changed),
+                                            screen);
+
   gtk_widget_show (screen->priv->term);
 }
 
@@ -255,6 +263,15 @@ terminal_screen_class_init (TerminalScreenClass *klass)
                   NULL, NULL,
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);  
+
+  signals[ENCODING_CHANGED] =
+    g_signal_new ("encoding_changed",
+                  G_OBJECT_CLASS_TYPE (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (TerminalScreenClass, encoding_changed),
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);  
 }
 
 static void
@@ -481,9 +498,9 @@ rebuild_title  (TerminalScreen *screen)
 }
 
 static void
-profile_changed_callback (TerminalProfile          *profile,
-                          TerminalSettingMask       mask,
-                          TerminalScreen           *screen)
+profile_changed_callback (TerminalProfile           *profile,
+                          const TerminalSettingMask *mask,
+                          TerminalScreen            *screen)
 {
   reread_profile (screen);
 }
@@ -1002,10 +1019,17 @@ static void
 new_window_callback (GtkWidget      *menu_item,
                      TerminalScreen *screen)
 {
+  char *name;
+
+  name = gdk_screen_make_display_name (gtk_widget_get_screen (menu_item));
+  
   terminal_app_new_terminal (terminal_app_get (),
                              terminal_profile_get_for_new_term (screen->priv->profile),
                              NULL,
-                             FALSE, FALSE, NULL, NULL, NULL, NULL, NULL, 1.0);
+                             FALSE, FALSE, NULL, NULL, NULL, NULL, NULL, 1.0,
+                             NULL, name, -1);
+
+  g_free (name);
 }
 
 static void
@@ -1015,7 +1039,8 @@ new_tab_callback (GtkWidget      *menu_item,
   terminal_app_new_terminal (terminal_app_get (),
                              terminal_profile_get_for_new_term (screen->priv->profile),
                              screen->priv->window,
-                             FALSE, FALSE, NULL, NULL, NULL, NULL, NULL, 1.0);
+                             FALSE, FALSE, NULL, NULL, NULL, NULL, NULL, 1.0,
+                             NULL, NULL, -1);
 }
 
 static void
@@ -1691,6 +1716,13 @@ terminal_screen_widget_selection_changed (GtkWidget      *term,
   g_signal_emit (G_OBJECT (screen), signals[SELECTION_CHANGED], 0);
 }
 
+static void
+terminal_screen_widget_encoding_changed (GtkWidget      *term,
+                                         TerminalScreen *screen)
+{
+  g_signal_emit (G_OBJECT (screen), signals[ENCODING_CHANGED], 0);
+}
+
 static void
 title_entry_changed (GtkWidget      *entry,
                      TerminalScreen *screen)
